"""The app command."""


from json import dumps
from .base import Base
import ruamel.yaml

class Add_config(Base):
    """Cluster commands"""
    def run(self):
        self.append_launch_file(self.options['<app>'])




def gcp_commands(self):
    """This functions creates a variety of commands to augment the configuration of Kubernetes on the Google cloud platform.
    """
    lc=self.launch_config
    commands={}
    commands['create_service_account']="gcloud iam service-accounts create "+lc['g_service_account_name']+ " --display-name "+ lc['g_service_account_name']
    commands['create_key']="gcloud iam service-accounts keys create "+self.cwd+"/gcp/"+lc['g_authorization_file'] +" --iam-account "+lc['g_service_account_name']+"@"+lc['g_project']+".iam.gserviceaccount.com"
    commands['get_policy']="gcloud iam service-accounts get-iam-policy "+lc['g_service_account_name']+"@"+lc['g_project']+".iam.gserviceaccount.com --format json > "+self.cwd+"gcp/policy.json"
    commands['set_policy']="gcloud iam service-accounts set-iam-policy "+lc['g_service_account_name']+"@"+lc['g_project']+".iam.gserviceaccount.com "+self.cwd+"/gcp/policy.json"
    commands['login']="gcloud auth login"
    commands['login_sa']="gcloud auth activate-service-account --key-file "+self.cwd+"/gcp/"+ lc['g_authorization_file']
    commands['create_project']="gcloud projects create "+lc['g_project']+" --set-as-default"
    commands['set_project']="gcloud config set project "+lc['g_project']
    commands['set_zone']="gcloud config set compute/zone "+lc['g_zone']
    commands['create']="gcloud container clusters create "+lc['g_cluster_name']+" --num-nodes="+str(lc['g_num_nodes'])+" --machine-type="+lc['g_machine_type']+" --zone="+lc['g_zone']
    commands['get_credentials']="gcloud container clusters get-credentials "+lc['g_cluster_name']
    commands['stop']="gcloud container clusters resize "+lc['g_cluster_name']+" --size=0 --quiet"
    commands['normal_size']="gcloud container clusters resize "+lc['g_cluster_name']+" --size="+str(lc['g_num_nodes'])+" --quiet"
    commands['class_size']="gcloud container clusters resize "+lc['g_cluster_name']+" --size="+str(lc['g_num_nodes_class'])+" --quiet"
    commands['delete']="gcloud container clusters delete "+lc['g_cluster_name']+" --zone="+lc['g_zone']+" --quiet"
    commands['autoscale']="gcloud alpha container clusters update "+lc['g_cluster_name']+" --enable-autoscaling --min-nodes="+str(lc['g_num_nodes'])+" --max-nodes="+str(lc['g_max_nodes'])+" --zone="+lc['g_zone']+" --node-pool=default-pool"
    commands['create_fixedip']="gcloud compute addresses create "+lc['g_fixedip_namespace']+" --region="+lc['g_region']
    commands['describe_fixedip']="gcloud compute addresses describe "+lc['g_fixedip_namespace']+" --region="+lc['g_region']
    commands['delete_forwarding_rule']="gcloud compute forwarding-rules delete forwarding_rule --quiet"
    commands['delete_fixedip']="gcloud compute addresses delete "+lc['g_fixedip_namespace']+" --region="+lc['g_region']+" --quiet"
    commands['describe_cluster']="gcloud container clusters describe "+lc['g_cluster_name']
    #commands['backup_ssh']="mkdir "+self.cwd+"/.ssh &&"+ "cp ~/.ssh/id_rsa "+self.cwd+"/.ssh/id_rsa_"+lc['cluster_name']+"&& cp ~/.ssh/id_rsa.pub "+self.cwd+"/.ssh/id_rsa_"+lc['cluster_name']+".pub"
    return commands



def test():
    print("it workss")

def azure_commands(self):
    """This functions creates a variety of commands to augment the configuration of Kubernetes on the Google cloud platform.
    """
    lc=self.launch_config
    commands={}
    commands['login']="az login"
    commands['login_sa']="echo 'Service account login not yet available for Azure'"
    commands['create_project']="az group create --name="+lc['a_resource_group']+ " --location="+ lc['a_location']
    commands['set_project']="echo 'No need to set project in Azure.'"
    commands['set_zone']="echo 'No need to set project in Azure.'"
    commands['delete_project']="az group delete --name="+lc['a_resource_group']+" --yes --no-wait"
    commands['list-locations']="az account list-locations"
    commands['autoscale']="echo 'Autoscaling currently not possible.'"
    if lc['a_service_type']=="aks":
        commands['create_cluster']="az aks create --resource-group="+lc['a_resource_group'] +" --name="+lc['a_cluster_name']+" --agent-count="+str(lc['a_num_nodes'])+" --agent-vm-size="+lc['a_machine_type']+" --generate-ssh-keys --no-wait"
        commands['describe_cluster']="az aks list --resource-group="+lc['a_resource_group']
        commands['delete_cluster']="az aks delete --name="+lc['a_cluster_name']+" --resource-group="+lc['a_resource_group']+" --no-wait"
        commands['get_credentials']="az aks get-credentials --resource-group="+lc['a_resource_group']+" --name="+lc['a_cluster_name']
        commands['normal_size_cluster']="az aks scale  --agent-count "+str(lc['a_num_nodes']) +" --name="+lc['a_cluster_name']+" --resource-group="+lc['a_resource_group']
        commands['class_size_cluster']="az aks scale --agent-count "+ str(lc['a_num_nodes_class'])+" --name="+lc['a_cluster_name']+" --resource-group="+lc['a_resource_group']
    elif lc['a_service_type']=="acs":
        commands['create_cluster']="az acs create --orchestrator-type=kubernetes --resource-group="+lc['a_resource_group'] +" --name="+lc['a_cluster_name']+" --dns-prefix="+lc['a_dns_prefix']+" --agent-count="+str(lc['a_num_nodes'])+" --agent-vm-size="+lc['a_machine_type']+" --generate-ssh-keys --no-wait"
        commands['describe_cluster']="az acs list  --resource-group="+lc['a_resource_group']
        commands['delete_cluster']="az acs delete  --resource-group="+lc['a_resource_group']+" --name="+lc['a_cluster_name']+" --no-wait"
        commands['get_credentials']="az acs kubernetes get-credentials --resource-group="+lc['a_resource_group']+" --name="+lc['a_cluster_name']+" --ssh-key-file=~/.ssh/id_rsa_"+lc['a_cluster_name']
        commands['normal_size_cluster']="az acs scale --resource-group="+lc['a_resource_group']+" --name="+lc['a_cluster_name']+" --new-agent-count "+str(lc['a_num_nodes'])
        commands['class_size_cluster']="az acs scale --resource-group="+lc['a_resource_group']+" --name="+lc['a_cluster_name']+" --new-agent-count "+str(lc['a_num_nodes_class'])
    commands['install_helm']='helm init --client-only'
    # Can't size to 0 lc['a_stop_cluster']="az acs scale --resource-group="+lc['a_resource_group']+" --name="+lc['a_cluster_name']+" --new-agent-count 0"
    commands['create_storage']= "az storage account create --resource-group="+lc['a_resource_group']+" --location="+lc['a_location']+" --sku=Standard_LRS  --name="+lc['a_storage_account']+" --kind=Storage"
    commands['get_storage_key']="az storage account keys list --account-name="+lc['a_storage_account']+" --resource-group="+lc['a_resource_group']+" --output=json | jq .[0].value -r"
    commands['create_keyvault']="az keyvault create --name="+lc['a_cluster_name']+" --resource-group="+ lc['a_resource_group']+" --location="+lc['a_location']+" --enabled-for-template-deployment true"
    commands['backup_private_key']="az keyvault secret set --vault-name="+ lc['a_cluster_name']+ " --name=id-rsa-"+lc['a_cluster_name']+" --file=~/.ssh/id_rsa_"+lc['a_cluster_name']
    commands['backup_public_key']="az keyvault secret set --vault-name="+ lc['a_cluster_name']+ " --name=id-rsa"+lc['a_cluster_name']+"-pub --file=~/.ssh/id_rsa_"+lc['a_cluster_name']+".pub"
    commands['get_private_key']="az keyvault secret show --vault-name="+ lc['a_cluster_name']+ " --name=id-rsa-"+lc['a_cluster_name']
    commands['get_public_key']="az keyvault secret show --vault-name="+ lc['a_cluster_name']+ " --name=id-rsa"+lc['a_cluster_name']+"-pub"
    return commands



"""The app command."""


from json import dumps
import os
from .base import Base
import ruamel.yaml
import sys
from .apps.jupyter import *

class App(Base):
    """Cluster commands"""
    def run(self):
        self.check_launch_file()
        app_file=self.launch_dir+'/apps/'+self.options['<app>']+'.yaml'
        if os.path.isfile(app_file):
            if self.options['<app>']=='jupyter':
                jupyter_action(self)
        else:
            print("The application "+self.options['<app>']+"is not available yet. Use `kubel -h` to see available apps.")



"""
kubel

Usage:
  kubel -h | --help
  kubel -v | --version
  kubel init (azure|gcp) [--jupyter][--airflow][--force]
  kubel add_config <app>
  kubel app <app> <command> [--dry-run]
  kubel cluster <command> [--dry-run]

Options:
  -h --help                         Show this screen.
  -v --version                      Show version.
  -f --force                        Force.
  --jupyter                         Set config to include jupyter.
  --echo                            Only print the comand, don't execute.

Examples:
  kubel cluster new

Help:
  For help using this tool, please see the Github repository:
  https://github.com/kubelaunch/kubelaunch-cli
"""

from inspect import getmembers, isclass
from docopt import docopt
from . import __version__ as VERSION
from os.path import abspath, dirname, join, isfile
import os

def main():
    """Main CLI entrypoint."""
    import kubelaunch.commands as kc
    options = docopt(__doc__, version=VERSION)

    # Here we'll try to dynamically match the command the user is trying to run
    # with a pre-defined command class we've already created.
    for (k, v) in options.items():
          if hasattr(kc, k) and v:
            module = getattr(kc, k)
            kc = getmembers(module, isclass)
            command = [command[1] for command in kc if command[0] != 'Base'][0]
            command = command(options)
            command.run()



"""The app command."""

from json import dumps
import ruamel.yaml
import os
import sys
from .base import Base
from .clusters.azure import *
from .clusters.gcp import *


class Cluster(Base):
    """Cluster commands"""
    def run(self):
        self.check_launch_file()
        if self.launch_config['cluster_location']=='azure':
            print('Azure Cloud')
            self.cluster_commands=azure_commands(self)
        elif self.launch_config['cluster_location']=='gcp':
            #print('Google cloud')
            self.cluster_commands=gcp_commands(self)
        if self.options['<command>']=='list':
            print("Listing out available commands.")
            print(ruamel.yaml.dump(self.cluster_commands, sys.stdout, Dumper=ruamel.yaml.RoundTripDumper))
        elif self.options['<command>'] in self.cluster_commands:
            print(self.bash_command(self.options['<command>'],self.cluster_commands[self.options['<command>']]))



"""The base command."""

import os
from shutil import copyfile
import ruamel.yaml
import subprocess
from shutil import copyfile

class Base(object):
    """A base command."""
    def __init__(self, options, *args, **kwargs):
        self.options = options
        self.args = args
        self.base_dir=os.path.abspath(os.path.join(os.path.dirname(__file__), os.pardir))
        self.launch_dir=self.base_dir+'/data/launch_files'
        self.notebooks_dir=self.base_dir+'/data/notebooks'
        self.cwd=os.getcwd()
        self.launch_file = self.cwd+'/launch.yaml'
        self.launch_config = {}
        self.cluster_commands = {}
        self.app_commands = {}

    def append_launch_file(self, app):
        app_config=self.launch_dir+'/apps/'+app+'.yaml'
        if os.path.isfile(app_config):
            print('Adding configuration for '+app+' to launch.yaml.')
            config=self.load_yaml(app_config)
            ruamel.yaml.round_trip_dump(config, open(self.launch_file, 'a'))
        else:
            print('The configuration for the application ', app, 'is not available.' )
        copyfile(self.notebooks_dir+'/apps/'+app+'.ipynb', self.cwd+'/'+app+'.ipynb')

    def bash_command(self, command, syntax):
        if self.options['--dry-run']:
            print("Printing (not running) "+command+":\n", syntax)
        else:
            try:
                print("Executing "+command+":\n", syntax)
                result=subprocess.check_output(syntax, stderr=subprocess.STDOUT, shell=True)
                result=result.decode("utf-8")
                return result
            except subprocess.CalledProcessError as e:
                return(e.output.decode("utf-8"))

    def bash_command2(self, command, syntax):
        print("Executing "+command+":\n", syntax)
        popen = subprocess.Popen(syntax, stdout=subprocess.PIPE, universal_newlines=True)
        for stdout_line in iter(popen.stdout.readline, ""):
            yield stdout_line
        popen.stdout.close()
        return_code = popen.wait()
        if return_code:
            raise subprocess.CalledProcessError(return_code, cmd)
        return

    def check_launch_file(self, key=None):
        if os.path.exists(self.launch_file):
            self.launch_config=self.load_yaml(self.launch_file)
            if key !=None and key not in self.launch_config:
                print('That application is not in the launch.yaml file.')
                quit()
        else:
            print("The launch.yaml file is not present in curently directory.  Run 'kubel init' to create it. ")
            quit()

    def check_overwrite_launch_file(self):
        if os.path.isfile(self.launch_file) and not self.options['--force']:
            print('The launch.yaml file already exists in the current directory. Add the --force flag to overwrite.' )
            quit()
        else:
            return True

    def load_yaml(self, file):
        with open(file, 'r') as yaml:
            kwargs=ruamel.yaml.round_trip_load(yaml, preserve_quotes=True)
        return kwargs

    def message(self, app):
        print('Adding configuration for '+app+' to launch.yaml.')

    def run(self):
        raise NotImplementedError('You must implement the run() method yourself!')



import ruamel.yaml
import sys
def jupyterhub_commands(self):
    lc=self.launch_config
    commands={}
    lc['jup_instance']=self.cwd+"/jupyter/"+lc['jup_namespace']
    lc['jup_config']=self.cwd+"/jupyter/"+lc['jup_namespace']+"/config.yaml"
    lc['jup_base']=self.cwd+"/jupyter/"+lc['jup_namespace']+"/values.yaml"
    #commands['install_ssl']="helm install --name=letsencrypt --namespace=kube-system stable/kube-lego --set config.LEGO_EMAIL="+lc['jup_email']+" --set config.LEGO_URL=https://acme-v01.api.letsencrypt.org/directory"
    commands['init']="This will do multiple steps to initialze Jupyter config."
    commands['show_config']="cat "+lc['jup_config']
    commands['clone_repo']="helm repo add jupyterhub "+lc['jup_helm_repo']+" && helm repo update"
    commands['install']="helm install jupyterhub/jupyterhub --version="+lc['jup_version']+" --name="+lc['jup_releasename']+" --namespace="+lc['jup_namespace']+" -f "+lc['jup_config']
    commands['upgrade']="helm upgrade "+lc['jup_namespace']+" jupyterhub/jupyterhub --version="+lc['jup_version']+" -f "+lc['jup_config']
    commands['describe']="kubectl --namespace="+lc['jup_namespace']+" get pod"
    commands['get_ip']="kubectl --namespace="+ lc['jup_namespace']+" get svc proxy-public"
    commands['delete_namespace']="kubectl delete namespace "+lc['jup_namespace']
    commands['delete']="helm delete "+lc['jup_releasename']+" --purge"
    commands['delete_namespace']="kubectl delete namespace "+lc['jup_namespace']
    #if lc['jup_ssl']:
    #    lc['jup_callback_url']= "https://"
    #else:
#        lc['jup_callback_url']= "http://"
#    lc['jup_callback_url']=lc['jup_callback_url']+lc['jup_url']+"/hub/oauth_callback"
    commands['jup_get_logs']="kubectl --namespace="+lc['jup_namespace']+" logs <insert_podname>"
    return commands

def jupyter_action(self):
    self.check_launch_file('jup_namespace')
    self.app_commands=jupyterhub_commands(self)
    if self.options['<command>']=='list':
        print("Listing out available commands.")
        print(ruamel.yaml.dump(self.app_commands, sys.stdout, Dumper=ruamel.yaml.RoundTripDumper))
    elif self.options['<command>']=='init':
        jupyter_init(self)
    elif self.options['<command>'][:4]=='jup_':
        if self.options['<command>'] in self.launch_config:
            print("setting config for ", self.options['<command>'])
            update_config(self,self.options['<command>'])
        else:
            print("Configuration ", self.options['<command>'], " was not found in the launchfile." )
    elif self.options['<command>'] in self.app_commands:
        #bash_command(self.options['<command>'], self.app_commands[self.options['<command>']], dry-run=False)
        print(self.bash_command(self.options['<command>'],self.app_commands[self.options['<command>']]))
    else:
        print("That command isn't available yet. Use `kubel app "+self.options['<command>']+" list` to see available commands.")
    return

def update_config(self, key):
    file=self.cwd+"/jupyter/"+self.launch_config['jup_namespace']+"/config.yaml"
    config = ruamel.yaml.round_trip_load(open(file), preserve_quotes=True)
    if key[4:] in config:
        print("updating config")
        config[key[4:]]=self.launch_config[key]
    else:
        print("inserting config")
        config.insert(len(config), key[4:], self.launch_config[key])
    ruamel.yaml.round_trip_dump(config, open(file, 'w'))
    return

def jupyter_init(self):
    lc=self.launch_config
    lc['jup_instance']=self.cwd+"/jupyter/"+lc['jup_namespace']
    lc['jup_config']=self.cwd+"/jupyter/"+lc['jup_namespace']+"/config.yaml"
    #lc['jup_base']=self.cwd+"/jupyter/"+lc['jup_namespace']+"/values.yaml"
    if lc['jup_rebuild_config']:
        print(self.bash_command('delete folder','rm -rf '+lc['jup_instance']))
        print(self.bash_command('create directory', 'mkdir jupyter'))
        print(self.bash_command('create directory', 'mkdir '+ lc['jup_instance']))
        #Random variable that will be added to the config file.
        self.bash_command("generate secret key", "openssl rand -hex 32 > "+ lc['jup_instance']+"/cookie_secret.txt")
        #Random variable that will be added to the config file.
        self.bash_command("generate secret token", "openssl rand -hex 32 > "+ lc['jup_instance']+"/secret_token.txt")
    #     #download reference config
        self.bash_command("get reference config", "wget -P "+ lc['jup_instance']+" "+lc['jup_config_init'])
        self.bash_command("move reference config","mv "+ lc['jup_instance']+"/values.yaml " +lc['jup_instance']+"/reference.yaml")
         #init_jupyterhub_config(lc)
        if lc['jup_set_fixed_ip']:
            inp = """    #These are the only two required fields we need to launch
             proxy:
               secretToken: null
               service:
                  loadBalancerIP: null
             hub:
                   cookieSecret: null
                 """
        else:
             inp = """    #These are the only two required fields we need to launch
             proxy:
               secretToken: null
             hub:
               cookieSecret: null
                 """
    #         #This will write out a basic .YAML file.
        with open(lc['jup_instance']+'/cookie_secret.txt', 'r') as f:
                   cookie_secret = f.read().rstrip()
        with open(lc['jup_instance']+'/secret_token.txt', 'r') as f:
                   secret_token = f.read().rstrip()
    #
        config = ruamel.yaml.load(inp, ruamel.yaml.RoundTripLoader)

        config['hub']['cookieSecret']=cookie_secret
        config['proxy']['secretToken']=secret_token
        if lc['jup_set_fixed_ip']:
            config['proxy']['service']['loadBalancerIP']=lc['jup_fixed_ip']
        if lc['cluster_location']=='azure':
            config['prePuller']=lc['jup_prePuller']
        ruamel.yaml.round_trip_dump(config, open(lc['jup_config'], 'w'))
        lc['jup_rebuild_config']=False
        ruamel.yaml.round_trip_dump(lc, open(self.launch_file, 'w'))
    else:
        print("Currently the launchfile 'jup_rebuild_config' parameter is set to false.")
    return



"""The init command initializes a configuration file in
   the users home directory."""

from .base import Base
import ruamel.yaml
import os.path
from shutil import copyfile


class Init(Base):
    """Initialize!"""
    def run(self):
        """"Run will copy a file from launch_files """
        self.check_overwrite_launch_file()
        if self.options['gcp']:
            cluster_location='gcp'
        elif self.options['azure']:
            cluster_location='azure'
        self.message(cluster_location)
        config=self.load_yaml(self.launch_dir+'/clusters/'+cluster_location+'.yaml')

        #Create the launch_file in the user directory
        ruamel.yaml.round_trip_dump(config, open(self.launch_file, 'w'))
        copyfile(self.notebooks_dir+'/clusters/'+cluster_location+'.ipynb', self.cwd+'/'+cluster_location+'.ipynb')
        copyfile(self.base_dir+'/data/readme.md', self.cwd+'/readme.md')

        if self.options['--jupyter']:
            print("Appending Jupyter")
            self.append_launch_file('jupyter')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""
from codecs import open
from os.path import abspath, dirname, join
from subprocess import call
from setuptools import Command, find_packages, setup
from kubelaunch import __version__

this_dir = abspath(dirname(__file__))

with open(join(this_dir, 'README.rst'), encoding='utf-8') as file:
    long_description = file.read()

class RunTests(Command):
    """Run all tests."""
    description = 'run tests'
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        """Run all tests!"""
        errno = call(['py.test', '--cov=kubelaunch', '--cov-report=term-missing'])
        raise SystemExit(errno)


setup(
    name = 'kubelaunch',
    version = '0.0.5',
    description = 'An opinionated AI stack built for Kubernetes by default',
    long_description = long_description,
    url = 'https://github.com/kuberlytics/kubelaunch-cli',
    author = 'Jason Kuruzovich',
    author_email = 'jkuruzovich@gmail.com',
    classifiers = [
        'Development Status :: 2 - Pre-Alpha',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6'
        ],
    keywords='kubernetes analytics jupyterhub airflow',
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),
    install_requires=['ruamel.yaml','docopt'],
    include_package_data=True,
    extras_require = {
        'test': ['coverage', 'pytest', 'pytest-cov'],
    },
    entry_points={
        'console_scripts': [
            'kubel=kubelaunch.cli:main'
        ],
    },
    cmdclass = {'test': RunTests},
)
